@@ -25,7 +25,6 @@
 #' @param samples Integer. Number of sampled networks from model's posterior predictive using \code{\link{simulate.mmsbm}}.
 #' @param new.data.dyad See \code{\link{simulate.mmsbm}}. Enables out-of-sample checking.
 #' @param new.data.monad See \code{\link{simulate.mmsbm}}. Enables out-of-sample checking.
-#' @param parametric_mm See \code{\link{simulate.mmsbm}}.
 #' @param seed See \code{\link{simulate.mmsbm}}.
 #' @param ... Currently ignored.
 #'
@@ -47,12 +46,17 @@
 #'                       data.monad = lazega_monadic,
 #'                       n.blocks = 2,
 #'                       mmsbm.control = list(seed = 123,
+#'                                            conv_tol = 1e-2,
 #'                                            hessian = FALSE))
 #' 
 #' ## Plot observed (red) and simulated (gray) distributions over 
 #' ## indegrees
 #' ## (typically a larger number of samples would be taken) 
+#' ## (strictly requires ggplot2)
+#' 
+#' \donttest{
 #' gof(lazega_mmsbm, gof_stat = "Indegree", samples = 2)
+#' }
 #'
 
 gof <- function (x, ...) {
@@ -67,7 +71,6 @@ gof.mmsbm <- function(x,
                       samples = 50,
                       new.data.dyad = NULL,
                       new.data.monad  = NULL, 
-                      parametric_mm = FALSE,
                       seed = NULL,
                       ...
                       ){
@@ -144,8 +147,7 @@ gof.mmsbm <- function(x,
 
   el <- simulate(x, samples, seed=seed,
                  new.data.dyad,
-                 new.data.monad,
-                 parametric_mm)
+                 new.data.monad)
   if(!is.null(new.data.dyad)){
     if(is.null(x$forms$timeID)){
       tid <- "(tid)"
