@@ -10,6 +10,7 @@
 #' each latent group by time period. "\code{effect}" provides a series of plots showing the estimated effect 
 #' of a shfit in monadic covariate values.
 #' @param FX with \code{type == "effect"}; a list resulting from a call to \code{covFX}.
+#' @param node with \code{type == "membership"}; a character string specifying the node for which group membership should be plotted.
 #' @param ... Currently ignored
 #'
 #' @return The requested plot object. 
@@ -33,6 +34,7 @@
 #'                       data.monad = lazega_monadic,
 #'                       n.blocks = 2,
 #'                       mmsbm.control = list(seed = 123,
+#'                                            conv_tol = 1e-2,
 #'                                            hessian = FALSE))
 #' 
 #' ## Plot blockmodel as network
@@ -41,7 +43,7 @@
 
 
 
-plot.mmsbm <- function(x, type="groups", FX=NULL, ...){ # network graph showing B-matrix
+plot.mmsbm <- function(x, type="groups", FX=NULL, node=NULL, ...){ # network graph showing B-matrix
   if(type %in% c("blockmodel", "membership", "hmm")){
     if (!requireNamespace("ggplot2", quietly = TRUE)) {
       stop("Package \"ggplot2\" needed to produce requested plot. Please install it.",
@@ -79,19 +81,79 @@ plot.mmsbm <- function(x, type="groups", FX=NULL, ...){ # network graph showing
     .bar.legend(colRamp, range(igraph::E(block.G)$weight))
   }
   
+  if(type=="blockmodel"){
+    x$dyadic.data$Y <- x$Y
+    nodes <- unique(x$monadic.data$`(nid)`)
+    MMat <- sapply(nodes,function(y){
+      Dsub <- x$dyadic.data[x$dyadic.data$`(sid)`==y | x$dyadic.data$`(rid)`==y,]
+      return(sapply(nodes, function(y){sum(Dsub$Y[Dsub$`(sid)`==y | Dsub$`(rid)`==y])}))
+    })
+    diag(MMat) <- 0
+    clusters <- head(x, n=length(nodes))
+    csort <- sort(sapply(nodes, function(y){which.max(sapply(clusters, "[[", y))}))
+    corder <- unlist(sapply(1:x$n_blocks, function(z){sort(clusters[[z]][names(csort)[csort==z]], decreasing=T)}))
+    MMat <- MMat[names(corder), names(corder)]
+    plot(1, 1,
+         xlim = c(.5, length(nodes) + .5),
+         ylim = c(.5, length(nodes) + .5),
+         main = "",
+         xlab = "",
+         ylab = "",
+         type = "n", axes = FALSE)
+    polygon.color <- c("white", "black")
+    for (i in 1:length(nodes)) {
+      for (t in 1:length(nodes)) {
+        temp <- ifelse(MMat[i,t] > 0, 2, 1)
+        polygon(c(.5 + t - 1, .5 + t, .5 + t, .5 + t - 1),
+                length(nodes) - c(i-.5, i-.5, i+.5, i+.5),
+                density = NA,
+                border = polygon.color[temp],
+                col = polygon.color[temp]) 
+      } 
+    }
+    par(xpd=FALSE)
+    for(i in 1:x$n_blocks){
+      if(i < x$n_blocks){
+        abline(h=length(nodes)-length(which(csort %in% 1:i))-.5, col="red", lty=2, lwd=2)
+        abline(v=length(which(csort %in% 1:i))+.5, col="red", lty=2, lwd=2)
+      }
+    }
+    
+    v.size <- rowMeans(x$MixedMembership)
+    bm <- x$BlockModel
+    bm[upper.tri(bm)] <- NA
+    dm <- data.frame(Sender = rep(paste("Group", 1:nrow(bm)), times = x$n_blocks),
+                     Receiver = rep(paste("Group", 1:nrow(bm)), each = x$n_blocks),
+                     Val = plogis(c(bm)))
+    dm <- dm[complete.cases(dm),]
+    dm$Sender  <- factor(dm$Sender, levels=rev(paste("Group", 1:nrow(bm))))
+    p <- ggplot2::ggplot(ggplot2::aes_string(y = "Sender", x = "Receiver", fill="Val"), data = dm) +
+      ggplot2::ggtitle("Edge Formation Between Blocs") + 
+      ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0.5)) +
+      ggplot2::geom_tile(color = "white") + 
+      ggplot2::theme_bw() +
+      ggplot2::scale_size(guide='none') +
+      ggplot2::scale_fill_gradient2(low = "#FEE0D2", mid = "#FB6A4A", high = "#99000D",
+                                    midpoint = max(dm$Val)/2, limit = c(0,max(dm$Val)), name="Edge\nProbability")
+    print(p)
+  }
+  
   if(type=="membership"){
-    avgmems <- lapply(1:nrow(x$MixedMembership), function(x){
-      tapply(x$MixedMembership[x,], x$monadic.data[,"(tid)"], mean)})
-    avgmems <- as.data.frame(cbind(rep(unique(as.character(x$monadic.data[,"(tid)"])), nrow(x$MixedMembership)),unlist(avgmems),
-                                   rep(1:nrow(x$MixedMembership), each=length(unique(x$monadic.data[,"(tid)"])))))
+    ifelse(is.null(node), 
+           nr <- 1:nrow(x$monadic.data),
+           nr <- which(x$monadic.data$`(nid)`==node))
+    avgmems <- lapply(1:nrow(x$MixedMembership), function(y){
+      tapply(x$MixedMembership[y,nr], x$monadic.data[nr,"(tid)"], mean)})
+    avgmems <- as.data.frame(cbind(rep(unique(as.character(x$monadic.data[nr,"(tid)"])), nrow(x$MixedMembership)),unlist(avgmems),
+                                   rep(1:nrow(x$MixedMembership), each=length(unique(x$monadic.data[nr,"(tid)"])))))
     colnames(avgmems) <- c("Time", "Avg.Membership", "Group")
     avgmems$Group <- factor(avgmems$Group, levels=length(unique(avgmems$Group)):1)
-    if(class(avgmems$Avg.Membership) == "factor"){avgmems$Avg.Membership <- as.numeric(as.character(avgmems$Avg.Membership))}
-    if(class(avgmems$Time) == "factor"){avgmems$Time <- as.numeric(as.character(avgmems$Time))}
+    if(class(avgmems$Avg.Membership) != "numeric"){avgmems$Avg.Membership <- as.numeric(as.character(avgmems$Avg.Membership))}
+    if(class(avgmems$Time) != "numeric"){avgmems$Time <- as.numeric(as.character(avgmems$Time))}
     return(ggplot2::ggplot() + 
-      ggplot2::geom_area(ggplot2::aes_string(y = "Avg.Membership", x = "Time", fill="Group"), data = avgmems,
-                stat="identity", position="stack") + 
-      ggplot2::guides(fill=ggplot2::guide_legend(title="Group")))
+             ggplot2::geom_area(ggplot2::aes_string(y = "Avg.Membership", x = "Time", fill="Group"), data = avgmems,
+                                stat="identity", position="stack") + 
+             ggplot2::guides(fill=ggplot2::guide_legend(title="Group")))
   }
   
   if(type=="effect"){
@@ -118,13 +180,13 @@ plot.mmsbm <- function(x, type="groups", FX=NULL, ...){ # network graph showing
   if(type=="hmm"){
     hms <- as.data.frame(do.call(rbind, lapply(1:nrow(x$Kappa), function(x){
       cbind(1:ncol(x$Kappa), x$Kappa[x,], x)
-      })))
+    })))
     colnames(hms) <- c("Time", "Kappa", "State")
     hms$State <- as.factor(hms$State)
     return(ggplot2::ggplot() + 
-      ggplot2::geom_area(ggplot2::aes_string(y = "Kappa", x = "Time", fill="State"), data = hms,
-                stat="identity", position="stack") + 
-      ggplot2::guides(fill=ggplot2::guide_legend(title="HMM State")))
+             ggplot2::geom_area(ggplot2::aes_string(y = "Kappa", x = "Time", fill="State"), data = hms,
+                                stat="identity", position="stack") + 
+             ggplot2::guides(fill=ggplot2::guide_legend(title="HMM State")))
   }
 }
 
